@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2007-8 Matteo Frigo
+ * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,22 +18,22 @@
  *
  */
 
-/* $Id: conf.c,v 1.19 2003/03/15 20:29:42 stevenj Exp $ */
 
 #include "dft.h"
 
 static const solvtab s =
 {
      SOLVTAB(X(dft_indirect_register)),
-     SOLVTAB(X(dft_rank0_register)),
+     SOLVTAB(X(dft_indirect_transpose_register)),
      SOLVTAB(X(dft_rank_geq2_register)),
      SOLVTAB(X(dft_vrank_geq1_register)),
-     SOLVTAB(X(dft_vrank2_transpose_register)),
-     SOLVTAB(X(dft_vrank3_transpose_register)),
      SOLVTAB(X(dft_buffered_register)),
      SOLVTAB(X(dft_generic_register)),
      SOLVTAB(X(dft_rader_register)),
+     SOLVTAB(X(dft_bluestein_register)),
      SOLVTAB(X(dft_nop_register)),
+     SOLVTAB(X(ct_generic_register)),
+     SOLVTAB(X(ct_genericbuf_register)),
      SOLVTAB_END
 };
 
@@ -41,10 +41,6 @@ void X(dft_conf_standard)(planner *p)
 {
      X(solvtab_exec)(s, p);
      X(solvtab_exec)(X(solvtab_dft_standard), p);
-     X(solvtab_exec)(X(solvtab_dft_inplace), p);
-#if HAVE_K7
-     X(solvtab_exec)(X(solvtab_dft_k7), p);
-#endif
 #if HAVE_SIMD
      X(solvtab_exec)(X(solvtab_dft_simd), p);
 #endif
