@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2007-8 Matteo Frigo
+ * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,11 @@
 #undef ST
 #define ST STA
 
+#define VTW VTW1
+#define TWVL TWVL1
+#define BYTW BYTW1
+#define BYTWJ BYTWJ1
+
 #define GENUS X(dft_t1bsimd_genus)
 extern const ct_genus GENUS;
 
