@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003, 2007-11 Matteo Frigo
- * Copyright (c) 2003, 2007-11 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2007-8 Matteo Frigo
+ * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  */
 
-#include SIMD_HEADER
+#include "simd.h"
 
 #undef LD
 #define LD LDA
@@ -30,6 +30,6 @@
 #define BYTW BYTW2
 #define BYTWJ BYTWJ2
 
-#define GENUS XSIMD(dft_t2bsimd_genus)
+#define GENUS X(dft_t2bsimd_genus)
 extern const ct_genus GENUS;
 
