@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2003 Matteo Frigo
- * Copyright (c) 2003 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2007-8 Matteo Frigo
+ * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
  *
  */
 
-/* $Id: vrank-geq1.c,v 1.41 2003/04/15 19:03:20 stevenj Exp $ */
 
 
 /* Plans for handling vector transform loops.  These are *just* the
@@ -47,16 +46,16 @@ typedef struct {
      plan_dft super;
 
      plan *cld;
-     int vl;
-     int ivs, ovs;
+     INT vl;
+     INT ivs, ovs;
      const S *solver;
 } P;
 
 static void apply(const plan *ego_, R *ri, R *ii, R *ro, R *io)
 {
      const P *ego = (const P *) ego_;
-     int i, vl = ego->vl;
-     int ivs = ego->ivs, ovs = ego->ovs;
+     INT i, vl = ego->vl;
+     INT ivs = ego->ivs, ovs = ego->ovs;
      dftapply cldapply = ((plan_dft *) ego->cld)->apply;
 
      for (i = 0; i < vl; ++i) {
@@ -65,10 +64,10 @@ static void apply(const plan *ego_, R *ri, R *ii, R *ro, R *io)
      }
 }
 
-static void awake(plan *ego_, int flg)
+static void awake(plan *ego_, enum wakefulness wakefulness)
 {
      P *ego = (P *) ego_;
-     AWAKE(ego->cld, flg);
+     X(plan_awake)(ego->cld, wakefulness);
 }
 
 static void destroy(plan *ego_)
@@ -81,7 +80,7 @@ static void print(const plan *ego_, printer *p)
 {
      const P *ego = (const P *) ego_;
      const S *s = ego->solver;
-     p->print(p, "(dft-vrank>=1-x%d/%d%(%p%))",
+     p->print(p, "(dft-vrank>=1-x%D/%d%(%p%))",
  	      ego->vl, s->vecloop_dim, ego->cld);
 }
 
@@ -93,18 +92,19 @@ static int pickdim(const S *ego, const tensor *vecsz, int oop, int *dp)
 
 static int applicable0(const solver *ego_, const problem *p_, int *dp)
 {
-     if (DFTP(p_)) {
-          const S *ego = (const S *) ego_;
-          const problem_dft *p = (const problem_dft *) p_;
+     const S *ego = (const S *) ego_;
+     const problem_dft *p = (const problem_dft *) p_;
 
-          return (1
-                  && FINITE_RNK(p->vecsz->rnk)
-                  && p->vecsz->rnk > 0
-                  && pickdim(ego, p->vecsz, p->ri != p->ro, dp)
-	       );
-     }
+     return (1
+	     && FINITE_RNK(p->vecsz->rnk)
+	     && p->vecsz->rnk > 0
 
-     return 0;
+	     /* do not bother looping over rank-0 problems,
+		since they are handled via rdft */
+	     && p->sz->rnk > 0
+
+	     && pickdim(ego, p->vecsz, p->ri != p->ro, dp)
+	  );
 }
 
 static int applicable(const solver *ego_, const problem *p_, 
@@ -136,12 +136,7 @@ static int applicable(const solver *ego_, const problem *p_,
 		    return 0;
 	  }
 
-	  /* Heuristic: don't use a vrank-geq1 for rank-0 vrank-1
-	     transforms, since this case is better handled by rank-0
-	     solvers. */
-	  if (p->sz->rnk == 0 && p->vecsz->rnk == 1) return 0;
-
-	  if (NONTHREADED_ICKYP(plnr)) return 0; /* prefer threaded version */
+	  if (NO_NONTHREADEDP(plnr)) return 0; /* prefer threaded version */
      }
 
      return 1;
@@ -186,14 +181,16 @@ static plan *mkplan(const solver *ego_, const problem *p_, planner *plnr)
      X(ops_zero)(&pln->super.super.ops);
      pln->super.super.ops.other = 3.14159; /* magic to prefer codelet loops */
      X(ops_madd2)(pln->vl, &cld->ops, &pln->super.super.ops);
-     pln->super.super.pcost = pln->vl * cld->pcost;
+
+     if (p->sz->rnk != 1 || (p->sz->dims[0].n > 64))
+	  pln->super.super.pcost = pln->vl * cld->pcost;
 
      return &(pln->super.super);
 }
 
 static solver *mksolver(int vecloop_dim, const int *buddies, int nbuddies)
 {
-     static const solver_adt sadt = { mkplan };
+     static const solver_adt sadt = { PROBLEM_DFT, mkplan, 0 };
      S *slv = MKSOLVER(S, &sadt);
      slv->vecloop_dim = vecloop_dim;
      slv->buddies = buddies;
@@ -208,7 +205,7 @@ void X(dft_vrank_geq1_register)(planner *p)
      /* FIXME: Should we try other vecloop_dim values? */
      static const int buddies[] = { 1, -1 };
 
-     const int nbuddies = sizeof(buddies) / sizeof(buddies[0]);
+     const int nbuddies = (int)(sizeof(buddies) / sizeof(buddies[0]));
 
      for (i = 0; i < nbuddies; ++i)
           REGISTER_SOLVER(p, mksolver(buddies[i], buddies, nbuddies));
