@@ -1,7 +1,7 @@
 (*
  * Copyright (c) 1997-1999 Massachusetts Institute of Technology
- * Copyright (c) 2003, 2006 Matteo Frigo
- * Copyright (c) 2003, 2006 Massachusetts Institute of Technology
+ * Copyright (c) 2003, 2007-8 Matteo Frigo
+ * Copyright (c) 2003, 2007-8 Massachusetts Institute of Technology
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *)
-(* $Id: gen_twidsq_c.ml,v 1.8 2006-02-12 23:34:12 athena Exp $ *)
 
 open Util
 open Genutil
 open C
 
-let cvsid = "$Id: gen_twidsq_c.ml,v 1.8 2006-02-12 23:34:12 athena Exp $"
 type ditdif = DIT | DIF
 let ditdif = ref DIT
 
@@ -32,9 +30,9 @@ let usage = "Usage: " ^ Sys.argv.(0) ^ " -n <number> [ -dit | -dif ]"
 
 let reload_twiddle = ref false
 
-let uistride = ref Stride_variable
-let uvstride = ref Stride_variable
-let udist = ref Stride_variable
+let urs = ref Stride_variable
+let uvs = ref Stride_variable
+let ums = ref Stride_variable
 
 let speclist = [
   "-dit",
@@ -49,41 +47,38 @@ let speclist = [
   Arg.Unit(fun () -> reload_twiddle := true),
   " do not collect common twiddle factors";
 
-  "-with-istride",
-  Arg.String(fun x -> uistride := arg_to_stride x),
+  "-with-rs",
+  Arg.String(fun x -> urs := arg_to_stride x),
   " specialize for given input stride";
 
-  "-with-vstride",
-  Arg.String(fun x -> uvstride := arg_to_stride x),
+  "-with-vs",
+  Arg.String(fun x -> uvs := arg_to_stride x),
   " specialize for given vector stride";
 
-  "-with-dist",
-  Arg.String(fun x -> udist := arg_to_stride x),
-  " specialize for given dist"
+  "-with-ms",
+  Arg.String(fun x -> ums := arg_to_stride x),
+  " specialize for given ms"
 ]
 
 let generate n =
   let rioarray = "x"
-  and istride = "is"
-  and vstride = "vs"
+  and rs = "rs" and vs = "vs"
   and twarray = "W" 
-  and i = "i" 
-  and m = "m"
-  and dist = "dist" in
+  and m = "m" and mb = "mb" and me = "me" and ms = "ms" in
 
   let sign = !Genutil.sign 
   and name = !Magic.codelet_name 
   and byvl x = choose_simd x (ctimes (CVar "VL", x)) 
-  and bytwvl x = choose_simd x (ctimes (CVar "TWVL", x)) in
+  and bytwvl x = choose_simd x (ctimes (CVar "TWVL", x)) 
+  and bytwvl_vl x = choose_simd x (ctimes (CVar "(TWVL/VL)", x)) in
   let ename = expand_name name in
 
-  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy true in
+  let (bytwiddle, num_twiddles, twdesc) = Twiddle.twiddle_policy 0 true in
   let nt = num_twiddles n in
 
-  let svstride = either_stride (!uvstride) (C.SVar vstride)
-  and sistride = either_stride (!uistride) (C.SVar istride) in
-  let _ = Simd.ovs := stride_to_string "dist" !udist in
-  let _ = Simd.ivs := stride_to_string "dist" !udist in
+  let svs = either_stride (!uvs) (C.SVar vs)
+  and srs = either_stride (!urs) (C.SVar rs) in
+  let sms = stride_to_string "ms" !ums in
 
   let byw =
     if !reload_twiddle then
@@ -97,14 +92,14 @@ let generate n =
 
   let ioi = 
     locative_v_array_c n n 
-      (C.varray_subscript rioarray svstride sistride) 
-      (C.varray_subscript "BUG" svstride sistride) 
-      locations 
+      (C.varray_subscript rioarray svs srs) 
+      (C.varray_subscript "BUG" svs srs) 
+      locations sms
   and ioo = 
     locative_v_array_c n n 
-      (C.varray_subscript rioarray svstride sistride) 
-      (C.varray_subscript "BUG" svstride sistride) 
-      locations 
+      (C.varray_subscript rioarray svs srs) 
+      (C.varray_subscript "BUG" svs srs) 
+      locations sms
   in
 
   let lioi = load_v_array_c n n ioi in
@@ -118,35 +113,41 @@ let generate n =
   let odag = store_v_array_c n n ioo (transpose output) in
   let annot = standard_optimizer odag in
 
+  let vm = CVar m and vmb = CVar mb and vme = CVar me in
+
   let body = Block (
-    [Decl ("INT", i);
+    [Decl ("INT", m);
      Decl (C.realtypep, rioarray)],
     [Stmt_assign (CVar rioarray,
 		  CVar (if (sign < 0) then "ri" else "ii"));
-     For (Expr_assign (CVar i, Integer 0),
-	  Binop (" < ", CVar i, CVar m),
+     For (list_to_comma
+	    [Expr_assign (vm, vmb);
+	     Expr_assign (CVar twarray, 
+			  CPlus [CVar twarray; 
+				 ctimes (vmb, 
+					 bytwvl_vl (Integer nt))])],
+	  Binop (" < ", vm, vme),
 	  list_to_comma 
-	    [Expr_assign (CVar i, CPlus [CVar i; byvl (Integer 1)]);
+	    [Expr_assign (vm, CPlus [vm; byvl (Integer 1)]);
 	     Expr_assign (CVar rioarray, CPlus [CVar rioarray; 
-						byvl (CVar !Simd.ivs)]);
+						byvl (CVar sms)]);
 	     Expr_assign (CVar twarray, CPlus [CVar twarray; 
 					       bytwvl (Integer nt)]);
-	     make_volatile_stride (CVar istride);
-	     make_volatile_stride (CVar vstride)
+	     make_volatile_stride (CVar rs);
+	     make_volatile_stride (CVar vs)
 	   ],
-	  Asch annot);
-     Return (CVar twarray)
-   ]) in
+	  Asch annot)]) in
 
   let tree = 
-    Fcn (("static " ^ C.constrealtypep), ename,
+    Fcn (("static void"), ename,
 	 [Decl (C.realtypep, "ri");
 	  Decl (C.realtypep, "ii");
 	  Decl (C.constrealtypep, twarray);
-	  Decl (C.stridetype, istride);
-	  Decl (C.stridetype, vstride);
-	  Decl ("INT", m);
-	  Decl ("INT", dist)],
+	  Decl (C.stridetype, rs);
+	  Decl (C.stridetype, vs);
+	  Decl ("INT", mb);
+	  Decl ("INT", me);
+	  Decl ("INT", ms)],
          add_constants body)
   in
   let twinstr = 
@@ -157,9 +158,9 @@ let generate n =
     Printf.sprintf
       "static const ct_desc desc = {%d, %s, twinstr, &GENUS, %s, %s, %s, %s};\n\n"
       n (stringify name) (flops_of tree) 
-      (stride_to_solverparm !uistride) 
-      (stride_to_solverparm !uvstride)
-      (stride_to_solverparm !udist) 
+      (stride_to_solverparm !urs) 
+      (stride_to_solverparm !uvs)
+      (stride_to_solverparm !ums) 
 
   and register = 
     match !ditdif with
@@ -174,7 +175,7 @@ let generate n =
     (Printf.sprintf "{\n%s(p, %s, &desc);\n}" register ename)
   in
 
-  (unparse cvsid tree) ^ "\n" ^ init
+  (unparse tree) ^ "\n" ^ init
 
 
 let main () =
